/*
 *   @(#) $Id: BaseIoConnector.java 372232 2006-01-25 15:01:12Z trustin $
 *
 *   Copyright 2004 The Apache Software Foundation
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 */
package org.apache.mina.common.support;

import java.net.SocketAddress;

import org.apache.mina.common.ConnectFuture;
import org.apache.mina.common.IoConnector;
import org.apache.mina.common.IoFilterChainBuilder;
import org.apache.mina.common.IoHandler;

/**
 * A base implementation of {@link IoConnector}.
 * 
 * @author The Apache Directory Project (dev@directory.apache.org)
 * @version $Rev: 372232 $, $Date: 2006-01-26 00:01:12 +0900 (Thu, 26 Jan 2006) $
 */
public abstract class BaseIoConnector extends BaseIoService implements IoConnector
{
    private int defaultConnectTimeout = 60; // 1 minute

    protected BaseIoConnector()
    {
    }
    
    public ConnectFuture connect( SocketAddress address, IoHandler handler )
    {
        return connect( address, handler, IoFilterChainBuilder.NOOP );
    }

    public ConnectFuture connect( SocketAddress address, SocketAddress localAddress, IoHandler handler )
    {
        return connect( address, localAddress, handler, IoFilterChainBuilder.NOOP );
    }

    public int getConnectTimeout()
    {
        return defaultConnectTimeout;
    }

    public long getConnectTimeoutMillis()
    {
        return defaultConnectTimeout * 1000L;
    }

    public void setConnectTimeout( int defaultConnectTimeout )
    {
        if( defaultConnectTimeout <= 0 )
        {
            throw new IllegalArgumentException( "defaultConnectTimeout: " + defaultConnectTimeout );
        }
        this.defaultConnectTimeout = defaultConnectTimeout;
    }    
}
